class URL(object):
    SANDBOX_BASE_URL = 'https://stg-api.sandbox.paypay.ne.jp'
    PRODUCTION_BASE_URL = 'https://api.paypay.ne.jp'
    CODE = "/v2/codes"
    PAYMENT = "/v2/payments"
    ACCOUNT_LINK = "/v1/qr/sessions"
